/* Iterator objects */

#include <Python.h>
#include "xdatatype.h"
#include "xlistobject.h"
#include "kernel.h"

#define DEFERRED_ADDRESS(ADDR) 0


static int 
init_type(PyObject* cls) {
	char* facets_c;
	int facets_l;
	int size;
	int i;
	PyObject* xml_fun_name;
	PyObject* py_fun_name;
	PyObject* facet;
	PyObject* method;
	PyObject* xml_methods;
	PyObject* py_methods;
	PyObject* facets;
	PyObject* tag;
	PyTypeObject* tcls = (PyTypeObject*) cls;

	PySys_WriteStdout("cf 1");
	xml_methods = PyDict_GetItemString(tcls->tp_dict, "__xml_methods__");
	if (xml_methods)
		return 0;

	PySys_WriteStdout("cf 2");
	xml_methods = PyDict_New();
	py_methods = PyDict_New();

	PyDict_SetItemString(tcls->tp_dict, "__xml_methods__", xml_methods);
	PyDict_SetItemString(tcls->tp_dict, "__py_methods__", py_methods);
	
	facets = PyObject_GetAttrString(cls, "__facets__");

	if (facets == NULL || facets == Py_None) {
		PyErr_Clear();
		return 0;
	}

	PySys_WriteStdout("cf 3");

	if (PyString_Check(facets)) {
		PyString_AsStringAndSize(facets, &facets_c, &facets_l);
		facets = kernel_xml2seq_c(facets_c, facets_l, NULL);
		PySys_WriteStdout("cf 4");
		if (facets == NULL) {
			return -1;
		}
	}
	else
		return -1;


	size = PyList_GET_SIZE(facets);
	PySys_WriteStdout("elementi in facets %s = %i", facets_c, size);
	for (i = 0; i < size; ++i) {
		facet = PyList_GET_ITEM(facets, i);
		if (!PyList_Check((PyObject*)facet))
			continue;
		
		tag = PyXList_GET_TAG(facet);
		if (tag == NULL)
			return -1;

		xml_fun_name = PyString_FromString("xml_");
		PyString_Concat(&xml_fun_name, tag);
		PySys_WriteStdout(PyString_AsString(xml_fun_name));
		method = PyObject_GetAttr(cls, xml_fun_name);
		if (method) {
			PyDict_SetItem(xml_methods, (PyObject*)facet, method);
		}

		py_fun_name = PyString_FromString("py_");
		PyString_Concat(&xml_fun_name, tag);
		method = PyObject_GetAttr(cls, py_fun_name);
		if (method) {
			PyDict_SetItem(py_methods, (PyObject*)facet, method);
		}
	}
	return 0;
}

static int
xdatatype_init(PyObject *self, PyObject *args, PyObject *kwds) {
	return init_type((PyObject*) self->ob_type);
}


static PyObject* 
xml_check(PyObject* self, PyObject* args) {
	int pos;
	PyObject* facet;
	PyObject* method;
	PyObject* xml_methods;
	PyObject* cls;
	PyObject* xml;
	PyTypeObject* tcls;

	if (!PyArg_ParseTuple(args, "O!O!", &PyType_Type, &cls, &PyString_Type, &xml))
		return NULL;

	tcls = (PyTypeObject*) cls;

	if (init_type(cls) == -1)
		return NULL;
	
	xml_methods = PyDict_GetItemString(tcls->tp_dict, "__xml_methods__");
	PySys_WriteStdout("ck 2");
	
	pos = 0;
	while (PyDict_Next(xml_methods, &pos, &facet, &method)) {
		PySys_WriteStdout("ck 3");
		xml = PyObject_CallFunctionObjArgs(method, facet, NULL);
		if (xml == NULL)
			return xml;
	}
	return xml;
}

static PyObject* 
py_check(PyObject* self, PyObject* args) {
	int pos;
	PyObject* facet;
	PyObject* method;
	PyObject* py_methods;
	PyObject* cls;
	PyObject* py;
	PyTypeObject* tcls;

	if (!PyArg_ParseTuple(args, "O!O", &PyType_Type, &cls, &py))
		return NULL;

	tcls = (PyTypeObject*) cls;

	if (init_type(cls) == -1)
		return NULL;
	
	py_methods = PyDict_GetItemString(tcls->tp_dict, "__py_methods__");
	PySys_WriteStdout("ck 2");
	
	pos = 0;
	while (PyDict_Next(py_methods, &pos, &facet, &method)) {
		py = PyObject_CallFunctionObjArgs(method, facet, NULL);
		if (py == NULL)
			return py;
	}
	return py;
}

static char next_doc[] = 
"it.next()->obj\n\n\
Get the next value, or raise StopIteration";

static char tell_doc[] = 
"it.tell()->int\n\n\
Return the iterator index in the original xlist. The index deals only with direct subitems\n\
of the original xlist";

static char seek_doc[] = 
"it.seek(pos)->int\n\n\
Set the iterator index at pos. Following objects are extracted starting from pos in the\n\
original xlist";


static PyMethodDef xdatatype_methods[] = {
	{"xml_check",	(PyCFunction)xml_check,	METH_VARARGS, next_doc	},
	{"py_check",		(PyCFunction)py_check,	METH_VARARGS, tell_doc	},
	{NULL,		NULL}		/* sentinel */
};

static char xdatatype_doc[] =
"please use the method xiter of a xlist or the operatore iter to get the iterator";

PyTypeObject PyXDataType_Type = {
	PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
	0,					/* ob_size */
	"xdatatype",				/* tp_name */
	sizeof(PyXDataTypeObject) + PyGC_HEAD_SIZE,	/* tp_basicsize */
	0,					/* tp_itemsize */
	/* methods */
	0, 		/* tp_dealloc */
	0,					/* tp_print */
	0,					/* tp_getattr */
	0,					/* tp_setattr */
	0,					/* tp_compare */
	0,					/* tp_repr */
	0,					/* tp_as_number */
	0,					/* tp_as_sequence */
	0,					/* tp_as_mapping */
	0,					/* tp_hash */
	0,					/* tp_call */
	0,					/* tp_str */
	PyObject_GenericGetAttr,		/* tp_getattro */
	0,					/* tp_setattro */
	0,					/* tp_as_buffer */
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,	/* tp_flags */
 	xdatatype_doc,					/* tp_doc */
 	0,		/* tp_traverse */
 	0,					/* tp_clear */
	0,					/* tp_richcompare */
	0,					/* tp_weaklistoffset */
	0,		/* tp_iter */
	0,		/* tp_iternext */
	xdatatype_methods,				/* tp_methods */
	0,					/* tp_members */
	0,					/* tp_getset */
	0,					/* tp_base */
	0,					/* tp_dict */
	0,					/* tp_descr_get */
	0,					/* tp_descr_set */
	0,											/* tp_dictoffset */
	(initproc)xdatatype_init,						/* tp_init */
	0, //PyType_GenericAlloc,						/* tp_alloc */
	0,									/* tp_new */
	0,//_PyObject_GC_Del,							/* tp_free */
};

void initxdatatype() {
	PyObject* method;
	PyXDataType_Type.ob_type = &PyType_Type;
	if (PyType_Ready(&PyXDataType_Type) < 0)
		return;

	method = Py_FindMethod(xdatatype_methods, (PyObject*)PyXDataType_Type.ob_type, "xml_check");
	method = PyClassMethod_New(method);
	PyDict_SetItemString(PyXDataType_Type.tp_dict, "xml_check", method);

	method = Py_FindMethod(xdatatype_methods, PyXDataType_Type.ob_type, "py_check");
	method = PyClassMethod_New(method);
	PyDict_SetItemString(PyXDataType_Type.tp_dict, "py_check", method);

	
}

