from __future__ import generators

import tempfile
import string
import StringIO
import re
from types import *
from struct import *
import copy
import operator
import sys
import binhex
import base64
import new
import imp

import time
from time import gmtime
from satine import xspace,xlist
from satine import xml2seq
from satine import kernel
from satine import strip

knull = 0
__attribute_flag__ = 0x4000

modules = {}


def replaceWhitespace(s):
	collapseWhitespace(s)
	words = s.split()
	s = ""
	for i in words:
		s += i+' '
	s = s.strip()
	
def collapseWhitespace(s):
	s = s.replace('\x00d',' ').replace('\x00a',' ').replace(s,'\x009',' ')

class _list(xlist):
    __tag__ = "list"
    __attrs__ = """<xsd:ID>id
                   <xsd:QName>itemType"""
    __items__ = """<xsd:annotation>?<xsd:simpleType>?"""

class union(xlist):
    __attrs__ = """<xsd:ID>id
                   <xsd:QName>memberType"""
    __items__ = """<xsd:annotation>?<xsd:simpleType>?"""

class length(xlist):
    __attrs__ = """<xsd:nonNegativeInteger>value
                   <xsd:boolean>fixed"""
    __items__ = """<xsd:annotation>?"""

class minLength(xlist):
    __attrs__ = """<xsd:nonNegativeInteger>value
                   <xsd:boolean default="false">fixed"""
    __items__ = """<xsd:annotation>?"""

class maxLength(xlist):
    __attrs__ = """<xsd:nonNegativeInteger>value
                   <xsd:boolean default="false">fixed"""
    __items__ = """<xsd:annotation>?"""

class pattern(xlist):
    __attrs__ = """<xsd:string>value"""
    __items__ = """<xsd:annotation>?"""

class enumeration(xlist):
    __attrs__ = """<xsd:string>value"""
    __items__ = """<xsd:annotation>?"""

class maxInclusive(xlist):
    __attrs__ = """<xsd:string>value
                   <xsd:boolean default="false">fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""

class maxExclusive(xlist):
    __attrs__ = """<xsd:string>value
                   <xsd:boolean default="false">fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""

class minInclusive(xlist):
    __attrs__ = """<xsd:string>value
                   <xsd:boolean default="false">fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""

class minExclusive(xlist):
    __attrs__ = """<xsd:string>value
                   <xsd:boolean default="false">fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""

class totalDigits(xlist):
    __attrs__ = """<xsd:positiveInteger>value
                   <xsd:boolean default="false">fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""

class fractionDigits(xlist):
    __attrs__ = """<xsd:nonNegativeInteger>value
                   <xsd:boolean default="false">fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""

class whitespace(xlist):
    __attrs__ = """<xsd:string>value
                   <xsd:boolean>fixed
                   <xsd:ID>id"""
    __items__ = """<xsd:annotation>?"""


class simpletype(xlist):
	__facets__ = ""

	def __init__(self):
		self.__init_type()
		
	def __init_type(cls):
		cls.xml_validate = []
		cls.py_validate = []
		if cls.__facets__ == "":
			return
		facets = strip(xml2seq(cls.__facets__))
		if facets is None:
			raise "invalid facets definition"
		cls.facets = None
		for facet in facets:
			xml_fun_name = "xml_"+facet.tpinfo.tag
			if hasattr(cls, xml_fun_name):
				xml_fun = getattr(cls, xml_fun_name)
				cls.xml_validate.append((xml_fun,facet))
			py_fun_name = "py_"+facet.tpinfo.tag
			if hasattr(cls, py_fun_name):
				py_fun = getattr(cls, py_fun_name)
				cls.py_validate.append((py_fun, facet))
	__init_type = classmethod(__init_type)
	
	def xml_check(self, xml):
		for fun, facet in self.xml_validate:
			xml = fun(facet, xml)
		return xml
	xml_check = classmethod(xml_check)

	def py_check(self, py):
		for fun, facet in self.py_validate:
			py = fun(facet, py)
		return py
	py_check = classmethod(py_check)

class string(simpletype):
	def xml_lenght(self, facet, xml):
		if len(xml) != facet.value:
			raise "wrong string length: string length should be %i, but it is %i" % (facet.value, len(xml))
		return xml
	xml_lenght = classmethod(xml_lenght)
	
	def xml_minLenght(self, facet, xml):
		if len(xml) < facet.value:
			raise "string length should be at least %i, but it is %i" % (facet.value, len(xml))
		return xml
	xml_minLenght = classmethod(xml_minLenght)

	def xml_maxLen(self, facet, xml):
		if len(xml) > facet.value:
			raise "string length should be at most %i, but it is %i" % (facet.value, len(xml))
		return xml
	xml_maxLen = classmethod(xml_maxLen)
	
	def xml_pattern(self, facet, xml):
		if re.match(facet.value, xml) == None:
			raise "string '%s' doesn't match the regular expression pattern %s" % (xml,facet.value)
		return xml
	xml_pattern = classmethod(xml_pattern)

	def xml_enumerate(self, facet, xml):
		self.enumeration.append(facet.value)
		return xml
	xml_enumerate = classmethod(xml_enumerate)

	def xml_whiteSpace(self, facet, xml):
		if facet.whiteSpace == 'preserve':
			return xml
		elif string.whiteSpace == 'replace':
			return replaceWhitespace(xml)
		elif string.whiteSpace == 'collapse':
			return collapseWhitespace(xml)
		else:
			raise "unknown whitespace policy %s" % (self.whiteSpace,)
	xml_whiteSpace = classmethod(xml_whiteSpace)

	def xml2py(self, xml):
		self.enumeration = []
		xml = self.xml_check(xml)
		if self.enumeration and xml not in self.enumeration:
			raise "value %s not in enumeration %s" (xml, repr(self.enumeration))
		return str(xml)

	def py2xml(self, py):
		xml = str(py)
		self.xml_check(xml)
		return xml

class normalizedString(string):
	__facets__ = """<xsd:pattern value="[\\S| ]*$">"""

class token(string): 
	__facets__ = """<xsd:pattern value="\\S*$">"""

class language(token): pass		
class NMTOKEN(token): pass
class Name(token): pass
class NCName(Name): pass

class ID(NCName): pass
class IDREF(NCName): pass
class ENTITY(NCName): pass
class anyURI(string): pass

class QName(string):
	def decode(self, xml):
		try:
			if type(xml) is TupleType:
				return xml
			w = xml.split(':')
			if len(w) == 1:
				return ("", xml)
			else:
				prefix,tag = w
				return (kernel.uris[prefix], tag)
		except:
			raise "qname %s is invalid (did you load the module for the prefix?)" % (xml,)
	decode = classmethod(decode)
		
class boolean(simpletype):
	def xml_pattern(self, facet, xml):
		if re.match(facet.value, xml) == None:
			raise "string '%s' doesn't match the regular expression pattern %s" % (xml,facet.value)
		return xml
	xml_pattern = classmethod(xml_pattern)

	def xml_whiteSpace(self, facet, xml):
		if facet.whiteSpace == 'preserve':
			return xml
		elif string.whiteSpace == 'replace':
			return replaceWhitespace(xml)
		elif string.whiteSpace == 'collapse':
			return collapseWhitespace(xml)
		else:
			raise "unknown whitespace policy %s" % (self.whiteSpace,)
	xml_whiteSpace = classmethod(xml_whiteSpace)

	def xml2py(self, xml):
		if xml == 0 or xml == 1:
			return xml
		
		self.xml_check(xml)
		xml = xml.lower()
		if xml == 'true':
			return 1
		elif xml == 'false':
			return 0
		else:
			raise "boolean value %s must be either true or false" % (xml,)
	xml2py = classmethod(xml2py)
		
			
	def py2xml(self, py):
		if type(py) is StringType:
			return py
		if py:
			return 'true'
		else:
			return 'false'
	py2xml = classmethod(py2xml)

class decimal(simpletype):
	def xml_totalDigits(self, facet, xml):
		if facet.value < len(xml):
			raise "Too many digits in number %s" % (xml,)
		return data
	xml_totalDigits = classmethod(xml_totalDigits)
		
	def xml_fractionDigits(self,facet, xml):
		fd = xml.len() - xml.find('.')
		if fd > xml.len():
			fd = 0
		if facet.value < fd:
			raise "Too many  fractional digits in number %s" % (xml,)
		return data
	xml_fractionDigits = classmethod(xml_fractionDigits)
				
	def xml_pattern(self, facet, xml):
		if re.match(facet.value, xml) == None:
			raise "string '%s' doesn't match the regular expression pattern %s" % (xml,facet.value)
		return xml
	xml_pattern = classmethod(xml_pattern)

	def py_maxInclusive(self, facet, py):
		if py > float(facet.value): 
			raise "maxInclusive: %f > %s" % (py, facet.value)
		return py
	py_maxInclusive = classmethod(py_maxInclusive)
			
	def py_maxExclusive(self, facet, py):
		if py >= float(facet.value): 
			raise "maxExclusive: %f >= %s" % (py, facet.value)
		return py
	py_maxExclusive = classmethod(py_maxExclusive)

	def py_minInclusive(self, facet, py):
		if data < float(facet.value): 
			raise "minInclusive: %f < %s" % (py, facet.value)
		return py
	py_minInclusive = classmethod(py_minInclusive)

	def py_minExclusive(self, facet, py):
		if py <= float(facet.value): 
			raise "minExclusive %f <= %s" % (py, facet.value)
		return py
	py_minExclusive = classmethod(py_minExclusive)
	
	def xml2py(self, xml):
		xml = str(xml)
		xml = self.xml_check(xml)
		py = float(xml)
		py = self.py_check(py)
		return res
	xml2py = classmethod(xml2py)

	def py2xml(self, py):
		py = self.py_check(py)
		return str(data)
	py2xml = classmethod(py2xml)

class integer(simpletype):
	def py_maxInclusive(self, facet, py):
		if py > int(facet.value): 
			raise "maxInclusive: %f > %s" % (py, facet.value)
		return py
	py_maxInclusive = classmethod(py_maxInclusive)
			
	def py_maxExclusive(self, facet, py):
		if py >= int(facet.value): 
			raise "maxExclusive: %f >= %s" % (py, facet.value)
		return py
	py_maxExclusive = classmethod(py_maxExclusive)

	def py_minInclusive(self, facet, py):
		if py < int(facet.value): 
			raise "minInclusive: %f < %s" % (py, facet.value)
		return py
	py_minInclusive = classmethod(py_minInclusive)

	def py_minExclusive(self, facet, py):
		if py <= int(facet.value): 
			raise "minExclusive %f <= %s" % (py, facet.value)
		return py
	py_minExclusive = classmethod(py_minExclusive)

	def xml2py(self, xml):
		py = long(xml)
		py = self.py_check(py)
		return py
	xml2py = classmethod(xml2py)

	def py2xml(self, py):
		py = self.py_check(py)
		return str(py)
	py2xml = classmethod(py2xml)


class nonPositiveInteger(integer):
	__facets__ = '<xsd:maxInclusive value="0">'

class negativeInteger(integer):
	__facets__ = '<xsd:maxExclusive value="0">'

class nonNegativeInteger(integer):
	__facets__ = '<xsd:minInclusive value="0">'

class positiveInteger(integer):
	__facets__ = '<xsd:minInclusive value="1">'

class unsignedLong(integer):
	__facets__ = '<xsd:minInclusive value="0"><xsd:maxInclusive value="18446744073709551615L">' 

class _long(integer):
	__tag__ = "long"
	__facets__ = '<xsd:minInclusive value="-9223372036854775808L"><xsd:maxInclusive value="9223372036854775807L">' 

class unsignedInt(unsignedLong):
	__facets__ = '<xsd:minInclusive value="0"><xsd:maxInclusive value="4294967295L">' 

class _int(integer):
	__tag__ = "int"
	
class short(_int):
	__facets__ = '<xsd:minInclusive value="-32768"><xsd:maxInclusive value="32767">' 

class byte(_int):
	__facets__ = '<xsd:minInclusive value="-128"><xsd:maxInclusive value="127">' 

class unsignedShort(_int):
	__facets__ = '<xsd:minInclusive value="0"><xsd:maxInclusive value="65535">' 

class unsignedByte(_int):
	__facets__ = '<xsd:minInclusive value="0"><xsd:maxInclusive value="255">' 

class _float(decimal):
	__tag__ = "float"

class _double(_float):
	__tag__ = "double"

class duration(xlist):
	pattern = None
	enumeration = None
	whiteSpace = None
	maxInclusive = None
	minInclusive = None
	maxExclusive = None
	minExclusive = None

	def xml2py(self, data):
		if type(data) is IntType:
			return data
		exp = re.match("(-*)P([0-9]*)Y([0-9]*)M([0-9]*)DT([0-9]*)H([0-9]*)M([0-9]*)S$", data)
		if exp == None:
			raise "unable to convert a tuple in a time";
		sign, years, months, days, hours, minutes, seconds = exp.groups()
		if sign == '-':
			return -time.mktime(years,months,days,hours,minutes,seconds,0,0,-1)
		else:
			return time.mktime(years,months,days,hours,minutes,seconds,0,0,-1)
	xml2py = classmethod(xml2py)

	def py2xml(self, data):
		if type(data) is StringType:
			return data
		if data > 0:
			return "P%iY%iM%iDT%iH%iN%i" % tuple(gmtime(data))[:6]
		else:
			return "-P%iY%iM%iDT%iH%iN%i" % tuple(gmtime(-data))[:6]
	py2xml = classmethod(py2xml)

	def validate_whiteSpacePolicy(self):
		if self.whiteSpace == 'collapse':
			collapseWhitespace(self.repr)
		elif self.whiteSpace == 'replace':
			replaceWhitespace(self.repr)

	def validate_pattern(self):
		if self.pattern.match(self.repr) == None:
			raise "The string doen't constraint to the regular expression pattern"
	
	def validate_enumeration(self):
		if self.repr not in self.enumeration:
			raise "The string doen't belong to the enumeration"

	def validate_whitespace(self):
		if self.whiteSpace == 'preserve':
			return
		elif self.whiteSpace == 'replace':
			return
		elif self.whiteSpace == 'collapse':
			return
		raise "The whiteSpace policy "+self.whiteSpace+" is not valid"

class dateTime:
	pattern = None
	enumeration = None
	whiteSpace = None
	maxInclusive = None
	minInclusive = None
	maxExclusive = None
	minExclusive = None

	def __init__(self, data):
		if type(data) is TupleType:
			try:
				self.year,self.month,self.day,self.hour,self.minute,self.second,dayofweek,dayofyear,dl = data
			except:
				raise "unable to convert a tuple in a time";
		elif type(data) is FloatType:
			try:
				self.years,self.months,self.days,self.hours,self.minutes,self.seconds,dayofweek,dayofyear,dl = time.localtime(data)
			except:
				raise "unable to convert a float in a time";
		elif (type(data) is StringType) or (type(data) is UnicodeType):
			self.year = 0
			self.month = 0
			self.day = 0
			self.hour = 0
			self.minute = 0
			self.second = 0
			self.zsign = 0
			self.zhour = 0
			self.zminute = 0
			self.UTC = False

			try:	
				d,t = self.data.split("T")
				self.year, self.month,self.day = map(int,d.split("-"))
				k = t.split(":")
				self.hour = int(k[0])
				self.minute = int(k[1])
				t = k[2]
				if t[-1] == 'Z':
					self.UTC = True
					self.second = float(t[0:-2])
				elif t.find('+'):
					s,z = t.split('+')
					self.zsign = 1
					self.zminute = int(z)
					self.zsecond = float(k[3])
					self.second = float(s)
				elif t.find('-'):
					s,z = t.split('-')
					self.zsign = -1
					self.zminute = int(z)
					self.zsecond = float(k[3])
					self.second = float(s)
				else:
					self.second = float(t)
			except:
				raise "incorrect data format "


	def __repr__(self):
		r = str(self.hour).zfill(2)
		r += ":"+str(self.minute).zfill(2)
		r += ":"+str(self.second).zfill(2)
		if self.UTC:
			r += "Z"
		elif self.zsign > 0:
			r += "+"+str(self.zminute).zfill(2)+str(self.zsecond).zfill(2)
		elif self.zsign < 0:
			r += "-"+str(self.zminute).zfill(2)+str(self.zsecond).zfill(2)
		return r

	def time(self):
		return time.mktime((self.year,self.month,self.day,self.hour,self.minute,self.second,0,0,-1))		
	
class time:
	pattern = None
	enumeration = None
	whiteSpace = None
	maxInclusive = None
	minInclusive = None
	maxExclusive = None
	minExclusive = None

	def __init__(self, data):
		if type(data) is TupleType:
			try:
				year,month,day,self.hour,self.minute,self.second,dayofweek,dayofyear,dl = data
			except:
				raise "unable to convert a tuple in a time"
		elif type(data) is FloatType:
			try:
				year,month,day,self.hour,self.minute,self.second,dayofweek,dayofyear,dl = time.localtime(data)
			except:
				raise "unable to convert a float in a time"
		elif (type(data) is StringType) or (type(data) is UnicodeType):
			self.hour = 0
			self.minute = 0
			self.second = 0
			self.zhour = 0
			self.zminute = 0
			self.UTC = False
			self.zsign = 0
			self.__applyWhiteSpacePolicy()
			self.__checkPattern()
			self.__checkEnumeration()
			self.__checkWhiteSpace()

			try:	
				k = self.data.split(":")
				self.hour = int(k[0])
				self.minute = int(k[1])
				t = k[2]
				if t[-1] == 'Z':
					self.UTC = True
					self.second = float(t[0:-2])
				elif t.find('+'):
					s,z = t.split('+')
					self.zsign = 1
					self.zminute = int(z)
					self.zsecond = float(k[3])
					self.second = float(s)
				elif t.find('-'):
					s,z = t.split('-')
					self.zsign = -1
					self.zminute = int(z)
					self.zsecond = float(k[3])
					self.second = float(s)
				else:
					self.second = float(t)
			except:
				raise "incorrect data format "+self.data


	def __repr__(self):
		r = str(self.hour).zfill(2)
		r += ":"+str(self.minute).zfill(2)
		r += ":"+str(self.second).zfill(2)
		if self.UTC:
			r += "Z"
		elif self.zsign > 0:
			r += "+"+str(self.zminute).zfill(2)+str(self.zsecond).zfill(2)
		elif self.zsign < 0:
			r += "-"+str(self.zminute).zfill(2)+str(self.zsecond).zfill(2)
		return r
	
	def time(self):
		return time.mktime((0,0,0,self.hour,self.minute,self.second,0,0,-1))		

class date:
	pattern = None
	enumeration = None
	whiteSpace = None
	maxInclusive = None
	minInclusive = None
	maxExclusive = None
	minExclusive = None

	def __init__(self, seq):
		if type(data) is TupleType:
			try:
				year,month,day,self.hour,self.minute,self.second,dayofweek,dayofyear,dl = data
			except:
				raise "unable to convert a tuple in a time"
		elif type(data) is FloatType:
			try:
				self.year,self.month,self.day,hour,minute,second,dayofweek,dayofyear,dl = time.localtime(data)
			except:
				raise "unable to convert a float in a time"
		elif (type(data) is StringType) or (type(data) is UnicodeType):
			self.year = 0
			self.month = 0
			self.day = 0
			self.zsign = 0
			self.zhour = 0
			self.zminute = 0
			self.UTC = False

			try:	
				d,t = self.data.split("T")
				self.year, self.month,self.day = map(int,d.split("-"))
				k = self.data.split("-")
				self.year = int(k[0])
				self.month = int(k[1])
				t = k[2]
				if t[-1] == 'Z':
					self.UTC = True
					self.day  = int(t[0:-2])
				elif t.find('+'):
					t,z = t.split('+')
					self.zsign = 1
					z1, z2 = z.split(':')
					self.zminute = int(z1)
					self.zsecond = float(z2)
					self.day = int(t)
				elif t.find('-'):
					t,z = t.split('-')
					self.zsign = -1
					z1, z2 = z.split(':')
					self.zminute = int(z1)
					self.zsecond = float(z2)
					self.day = int(t)
				else:
					self.day = int(t)
			except:
				raise "incorrect data format "+self.data


	def __repr__(self):
		r = str(self.hour).zfill(2)
		r += ":"+str(self.minute).zfill(2)
		r += ":"+str(self.second).zfill(2)
		if self.UTC:
			r += "Z"
		elif self.zsign > 0:
			r += "+"+str(self.zminute).zfill(2)+str(self.zsecond).zfill(2)
		elif self.zsign < 0:
			r += "-"+str(self.zminute).zfill(2)+str(self.zsecond).zfill(2)
		return r
	
	def time(self):
		return time.mktime((self.year,self.month,self.day,0,0,0,0,0,-1))		



class hexBinary(xlist):
	bufferThreasold = 128000
	def xml2py(self, data):
		ins = StringIO.StringIO(data)
		if len(data) > self.bufferThreasold:
			buf = tempfile.TemporaryFile()
		else:
			buf = StringIO.StringIO()
		binhex.hexbin(ins, buf)
	xml2py = classmethod(xml2py)
		
	def py2xml(self, data):
		out = StringIO.StringIO()
		binhex.binhex(data, out)
		return out.getvalue()
	py2xml = classmethod(py2xml)

	
class base64Binary(xlist):
	bufferThreasold = 128000
	def xml2py(self, data):
		ins = StringIO.StringIO(data)
		if len(data) > self.bufferThreasold:
			buf = tempfile.TemporaryFile()
		else:
			buf = StringIO.StringIO()
		base64.decode(ins, buf)
	xml2py = classmethod(xml2py)
		
	def py2xml(self, data):
		out = StringIO.StringIO()
		base64.encode(self.data, out)
		return out.getvalue()
	py2xml = classmethod(py2xml)

class dtunion(xlist):
	__facets__ = ""
	def xml2py(self, xml):
		realTypes = xml2seq(self.__facets__)
		if realTypes == None:
			raise "XML in itemTypes is not valid"
		for tp in realTypes:
			try:
				res = tp.xml2py(xml)
				return res
			except:
				pass
		raise "unable to assign value in union"
	xml2py = classmethod(xml2py)

	def py2xml(self, py):
		return str(py)
	py2xml = classmethod(py2xml)

class dtlist(xlist):
	__facets__ = ""
	separator = ","
	def xml2py(self, xml):
		rt = satine.xml2py(self.__facets__)
		if not rt:
			raise "unknown type %s" % (self.__facets__)
	
		res = []
		words = xml.split(self.separator)
		for i in words:
			res.append(rt.xml2py(i))
		return res
	xml2py = classmethod(xml2py)

	def py2xml(self, py):
		rt = satine.xml2py(self.itemType)
		if not rt:
			raise "unknown type %s" % (self.itemType)

		res = ""
		for i in py:
			res += rt.py2xml(i)+self.separator
	py2xml = classmethod(py2xml)
				


xspace(xsd = "http://www.w3.org/2001/XMLSchema")
