import satine
import sys
import string
import new
import time
from types import InstanceType, ListType, TupleType, DictType, StringType, UnicodeType, IntType, LongType, FileType
import re
from StringIO import StringIO
from satine.kernel import parsexml
import satine.kernel
import os


def search_p2u(self, file):
	"""search_pu2(file) -> dict\n\n scan a whole file and extract namespace definitions.
	Return a prefixes to uris map"""
	findprefix = re.compile(r"""xmlns="([^"]*)|xmlns:([^=]*)="([^"]*)""")
	p2u = {}
	oldpos = file.tell()
	file.seek(0)
	data = file.read()
	res = findprefix.findall(data)
	for (defuri, prefix, uri) in res:
		if defuri != "":
			p2u[''] = defuri
		else:
			p2u[prefix] = uri
	if not '' in p2u:
		p2u[''] = str(time.time())
	file.seek(oldpos)
	return p2u
		
def search_u2p(self, file):
	"""search_u2p(file) -> dict\n\n scan a whole file and extract namespace definitions.
	Return a uris to prefixes map"""
	findprefix = re.compile(r"""xmlns="([^"]*)|xmlns:([^=]*)="([^"]*)""")
	u2p = {}
	oldpos = file.tell()
	file.seek(0)
	data = file.read()
	res = findprefix.findall(data)
	for (defuri, prefix, uri) in res:
		if defuri != "":
			u2p[defuri] = ''
		else:
			u2p[prefix] = uri
	if not '' in u2p.values():
		u2p[str(time.time())] = ''
	return u2p

class reader:
	def __init__(self, file, p2u = None, criteria = None, style = 'xre'):
		if hasattr(file, 'mode'):
			if 'b' not in file.mode:
				raise "only binary stream are supported"
		self.file = file
		self.iterator = None
		self.pos = 0
		self.block = 2048
		self.p2u = [p2u, {}][p2u is None]
		self.criteria = criteria
		self.style = style
		
	def read(self, count):
		res = []
		for i in xrange(count):
			try:
				item = self.next()
			except:
				return res
			res.append(self.next())
		return res

	def write(self, data):
		raise "unsupported operation"
		
	def load(self):
		self.base = self.pos
		data = ""
		self.file.seek(self.base)
		self.res = satine.xlist()
		self.idx = 0
		while not self.res:
			buf = self.file.read(self.block)
			if not buf:
				raise StopIteration
			data += buf
			self.res, self.offsets = parsexml(data, self.p2u)
			if self.criteria:
				self.iterator = self.res.iter(self.criteria, self.style)
			else:
				self.iterator = self.res.iter()
		self.pos += self.offsets[-1]
			
	def seek(self, pos):
		self.base = pos
		self.load()

	def tell(self):
		return self.base + self.offsets[self.idx]
		
	def next(self):
		if not self.iterator:
			self.load()	
		while 1:
			try:
				item = self.iterator.next()
				self.idx = self.iterator.tell()
				return item
			except StopIteration:
				self.load()		

	def __iter__(self):
		return self
						  
class writer:
	def __init__(self, file, u2p = None):
		self.file = file
		self.u2p = [u2p, {}][u2p is None]
		
		self.stack = []
		self.declared_ns = []		

	def write(self, data):
		for item in data:
			self.writeOne(item)
		
	def writeOne(self, item):
		if satine.is_xlist(item):
			uri = item.tpinfo.uri
			xml = satine.py2xml(item, self.u2p, self.declared_ns)
			xml = xml[:-2]+'>\n'
			blankpos = xml.find(' ')
			pre = xml[1:blankpos-[0,1][blankpos==-1]]
			self.stack.append(pre)
			self.file.write(xml)
		elif item is None:
			if not self.stack:
				return
			xml = '</'+self.stack.pop()+'>\n'
			self.file.write(xml)
		else:
			self.file.write(item)
			self.file.write('\n')

	def seek(self, pos):
		raise "unsupported operation"
	
	def tell(self):
		return self.stream.tell()

	def iter(self):
		raise "unsupported operation"

"""
WORK IN PROGRESS
class xlistReaderWrapper(satine.xlist):
	"xlist that embeddes a file" 
	def __init__(self, stream):
		"build the xlist wrapper; stream contains the data"
		self.stream = stream
		self.new_modules = {}

"""
satine.xspace(satine="http://satine.sourceforge.net/schemas/kernel")

def file2py(filename):
	"""file2py(filename)-> xlist\n\nRead a file and convert its content to a string"""
	file = open(filename, 'rb')
	data = file.read()
	res = satine.xml2py(data)
	file.close()
	return res

def py2file(obj, filename, prefixes = None):
	"""py2file(filename[, prefixes])-> xlist\n\nWrite a xlist to a file"""
	data = satine.py2xml(obj, prefixes)

	file = open(filename, "bw")
	file.write("""<?xml version="1.0"?>\n""")
	file.write(data)
	file.close()
	return res

test_doc = """<addressbook>
           <person surname="Gates" name="Bill">The Microsoft chief</person>
           <person surname="Tolvald" name="Linus">Linux creator </person>
         </addressbook>"""
	
def test():
	print "read test\n"
	file = StringIO(test_doc)
	print "normal iteration"
	stream = reader(file)
	for item in stream:
		print "%i:" % stream.tell(), item
	print "filtered iteration"
	file = StringIO(test_doc)
	stream = reader(file, criteria = ".<person>|surname,name")
	for item in stream:
		print "%i:" % stream.tell(), item[0], item[1]

	
if __name__ == "__main__":
	test()
	