#!/usr/bin/env python

from distutils.core import setup
from distutils.core import Extension

description = """
XML Data Binding Technology for Python

Satine is a Python library that makes XML managment easy and complete. Satine 
  converts XML documents to Python lists with attributes (xlist). This technology 
  allows to:
- translate documents with namespaces, both in elements and attributes
- translate both documents without XMLSchema and documents with it. If the 
    XMLSchema is available, the document can be easily validated.
- random and partial access to XML documents
- work very fast. The data binding technology is coded in C.
"""
  
setup(name="Satine",
      version="1.0_beta-1",
      description=description,
      author="Francesco Garelli",
      author_email="garelli@acm.org",
      url="http://satine.sourceforge.net",
      license="LGPL",
      package_dir = {'': 'lib','kernel':'c/kernel'},
      data_files = [('lib/site-packages/satine/test', 
          ['lib/satine/test/addressbook1.xml','lib/satine/test/jungle.xml','lib/satine/test/simplesoap1.xml',
           'lib/satine/test/simplesoap2.xml','lib/satine/test/zoo.xml','lib/satine/test/2511B39.xsd',
           'lib/satine/test/xhtml.xsd'])],
      packages=['satine','satine.schemas','satine.grapes','satine.test','kernel'],
      ext_modules=[Extension("satine.kernel",["c/kernel/kernel.c","c/kernel/xiterobject.c",
                                           "c/kernel/xlistobject.c","c/kernel/xquery.c",
                                           "c/kernel/expat/expatll.c","c/kernel/expat/xmlrole.c",
                                           "c/kernel/expat/xmltok.c","c/kernel/xlistinfo.c",
                                           ],
                             include_dirs=["c/kernel/expat"],
                             define_macros=[('EXPATLL_EXPORTS',None)])
                    ]

     )

